﻿import os
from datetime import datetime
from enum import Enum

import win32com.client

from . import active_session

from . import _execptions


class sap_systems(Enum):
    V30 = "V30 [CALLCENTER]"
    V32 = "V32 [SPACE]"
    V35 = "V35 [SPACE]"
    WSP = "WSP [SPACE]"
    WSI = "WSI [SPACE]"
    WSD = "WSD [SPACE]"


_system: sap_systems
_user: str = ''
_pwd: str = ''
_client: str = "10"
_language: str = "DE"


def set_session_data(system_name: sap_systems = None, user_name: str = None, password: str = None, client: str = "10",
                     language: str = "DE"):
    if system_name is not None:
        global _system
        _system = system_name
    if user_name is not None:
        global _user
        _user = user_name
    if password is not None:
        global _pwd
        _pwd = password
    if client != "10":
        global _client
        _client = client
    if language != "DE":
        global _language
        _language = language


def destroy():
    active_session.session = None


def error():
    # Close the sap session and exit the application
    raise NotImplementedError


def _wait_for_sap_gui() -> win32com.client.CDispatch:
    is_open = False
    timer_init = datetime.now().timestamp()
    while not is_open:  # Es wird auf den Start vom SAP-Client bis zu 30 Sekunden gewartet
        try:
            if datetime.now().timestamp() - timer_init > 30:
                print('Cannot start SAP GUI')
            return win32com.client.GetObject("SAPGUI")
        except Exception as ex:
            print(f"SAP Gui still not found! ({ex})")
            is_open = False


def _open_sap_gui():
    os.system(
        f'start sapshcut.exe -system="{_system.value}" -sysname="{_system.value}" -language={_language} '
        f'-client={_client} -user={_user} -password={_pwd} -maxgui -type="Transaction" -command="SESSION_MANAGER" '
        f'-title="Automation"')


def _get_application(sap_gui_auto: win32com.client.CDispatch) -> win32com.client.CDispatch:
    if type(sap_gui_auto) != win32com.client.CDispatch:
        raise execptions.SapGuiInitFails

    application = sap_gui_auto.GetScriptingEngine
    if type(application) != win32com.client.CDispatch:
        raise execptions.SapGuiInitFails

    return application


def _get_connection(application: win32com.client.CDispatch) -> win32com.client.CDispatch:
    app_alive = False
    timer_init = datetime.now().timestamp()
    connection = None

    while not app_alive:
        try:
            if datetime.now().timestamp() - timer_init > 30:
                raise execptions.SapGuiInitFails
            connection = application.Children(0)
            app_alive = True
        except Exception as ex:
            print(f"{ex}")
            app_alive = False

    if type(connection) != win32com.client.CDispatch:
        raise execptions.SapGuiInitFails

    return connection


def _get_session(connection: win32com.client.CDispatch):
    session_alive = False
    timer_init = datetime.now().timestamp()

    while not session_alive:
        try:
            if datetime.now().timestamp() - timer_init > 30:
                raise execptions.SapGuiInitFails
            active_session.session = connection.Children(0)
            session_alive = True
        except Exception as ex:
            raise execptions.SapGuiInitFails from ex
    if type(active_session.session) != win32com.client.CDispatch:
        raise execptions.SapGuiInitFails

    print("Session startet")


def create():
    global _system, _language, _user, _pwd
    """
        SAP wird über sapshcut mit den SAP Credentials gestartet. Bei SSO wird IMMER der Login vom SSO-User genutzt,
        abweichende Login-Daten sind nicht möglich. Da müsste man so wie bei AA ein Login-Script bauen.
        Sollte bereits eine SAP Session existieren dann wird sich dort draufgehangen 
        und keine neue SAP-Instanz geöffnet.

        Parameter:

        Rückgabewerte:
            - Erfolgreich = True + Log in einer Liste
            - nicht Erfolgreich = False + Log in einer Liste
    """

    try:
        _create()
    except Exception as ex:
        print(f'Unbekannter Fehler beim Start von SAP ({ex})')


def _create():
    if _check_for_open_sessions():  # Existiert bereits eine Session wird sich mit der verbunden
        print("SAP Session existiert bereits, verbunden")
    if _preconditions() is False:
        raise execptions.SapGuiInitFails

    _open_sap_gui()

    sap_gui_auto = _wait_for_sap_gui()

    # Ab hier folgt grundsätzlich eine 1:1 Übersetzung aus den üblichen VBS Scripten
    # mit zusätzlichen Fehlerabfragen und Delays

    application = _get_application(sap_gui_auto)

    connection = _get_connection(application)

    _get_session(connection)


def _check_for_open_sessions():
    try:
        sap_gui = win32com.client.GetObject("SAPGUI")
        application = sap_gui.GetScriptingEngine
        connection = application.Children(0)
        active_session.session = connection.Children(0)
        return True
    except Exception as ex:
        print(ex)
        return False


def _preconditions():
    global _client, _language
    do_login = True
    if _system is None:
        do_login = False
        print("Kein System angegeben")

    if _user is None:
        print("Benutzername ist leer")
        do_login = False

    if _pwd is None:
        print("Passwort ist leer")
        do_login = False

    if _client == "":
        _client = "010"

    if _language == "":
        _language = "DE"

    if do_login:
        print("Need to write to log")
        # SAP.gui.Log.Ereignis("Anmeldung","Alle Vorbedingungen für die Anmeldung wurden erfüllt!");
    else:
        raise NotImplementedError
        # SAP.gui.Log.Ereignis("Anmeldung", "Nicht alle Vorbedingungen für die Anmeldungen wurden erfüllt!");

    return do_login


if __name__ == '__main__':
    raise NotImplementedError
